#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, sys
sys.path.insert(0, os.path.dirname(
	os.path.dirname(os.path.realpath(__file__)) ))
from django.core.management import setup_environ
import settings
setup_environ(settings)

import project.forager.settings as cfg


from optparse import OptionParser
parser = OptionParser(usage='%prog [options] MUSIC_PATH',
	description='List MUSIC_PATH, resolving each entry as an'
		' artist name, seeking newest albums for each, except for the'
		' ones inside and the ones added/listed before.')

parser.add_option('-c', '--count',
	action='store', dest='count',
	type='int', metavar='NUM', default=20,
	help='Number of artists to process, processed'
		' names are recorded to state file. Default: %defaults')
parser.add_option('-d', '--delay',
	action='store', dest='delay',
	type='int', metavar='SEC', default=0,
	help='Delay between artist release queries. Default: %defaults')
parser.add_option('--reindex',
	action='store_true', dest='reindex',
	help='Discard info on processed artists in state file and start update from scratch.')

parser.add_option('--debug',
	action='store_true', dest='debug',
	help='Give extra info on whats going on.')
optz,argz = parser.parse_args()
if len(argz) != 1: parser.error('This command takes exactly one argument')


# For the sake of "screen" or any other things that still believe in ascii
import codecs
codec = codecs.getwriter('utf-8')
sys.stdout = codec(sys.stdout)
sys.stderr = codec(sys.stderr)

import logging
logging.basicConfig(level=( logging.WARNING
	if not optz.debug else logging.DEBUG ))
log = logging.getLogger()


from project.forager.mbdb import update_releases
from time import sleep
import itertools as it, operator as op
import pickle


music_root = argz[0]

import fcntl
try:
	state = open(cfg.state_db, 'r+' if os.path.exists(cfg.state_db) else 'w+')
	fcntl.flock(state, fcntl.LOCK_EX | fcntl.LOCK_NB)
except (IOError, OSError) as err:
	raise OSError('Unable to acquire lockfile ({0})): {1}'.format(cfg.state_db, err))

def update_state(data):
	state.seek(0, os.SEEK_SET)
	state.truncate()
	pickle.dump(data, state)
	state.flush()


def path_feed(path_list, path_list_done, path_count=0):
	path_list, path_list_done = it.imap(set, (path_list, path_list_done))

	for path in sorted(path_list.difference(path_list_done)):
		yield path
		path_list_done.add(path)
		update_state(path_list_done)
		path_count += 1
		if path_count >= optz.count: break
		sleep(optz.delay)

	else:
		log.debug(u'Completed full-loop of path_list, restarting')
		# Will result in deep recursion if there's no limit or if it's too large
		for path in path_feed(path_list, list(), path_count): yield path


try: path_list_done = pickle.load(state)
except EOFError: path_list_done = set() # empty file was created

update_releases( music_root, path_list=path_feed( path_list_done=path_list_done,
	path_list=os.listdir(music_root) ) )

