import project.forager.settings as cfg
from django.db import models

from datetime import date
from email.utils import parsedate
from urllib import quote
from httplib import HTTPException

import operator as op


class MusicRelease(models.Model):

	title = models.CharField(max_length=256)
	artist = models.CharField(max_length=256)

	date = models.DateField(blank=True, null=True)
	link = models.CharField(max_length=512, blank=True, null=True)
	summary = models.TextField(blank=True, null=True)

	added = models.DateTimeField(auto_now_add=True, editable=False)

	# Basically it's a more reliable version of artist-album key,
	#  in case mbz updates album name
	mbdb_id = models.CharField(unique=True, max_length=256)

	def __unicode__(self): return u'{0} - {1}'.format(self.artist, self.title)

	class Meta(object):
		ordering = '-added',
		unique_together = 'artist', 'title' # no point listing the same album twice


	def get_absolute_url(self):
		if self.link: return self.link
		else:
			return u'http://en.wikipedia.org/w/index.php?title=Search'\
				u'&search={0}+album+{1}&go=Go'.format(
					*map(lambda x: quote(x.encode('utf-8')), (self.title, self.artist)) )


	def update_info(self, lfm_link):
		self.date, self.link, self.summary =\
			op.itemgetter('date', 'link', 'summary')(self._info(lfm_link))

	__info_cache = dict() # indexed by last.fm id as well as (artist,title) tuples
	def _info_cache(self, lfm_id=None, val=None):
		'Multi-key passthrough assignment'
		cache = MusicRelease.__info_cache
		if val is None: return cache[lfm_id or (self.artist, self.title)]
		else:
			if lfm_id: cache[lfm_id] = val
			cache[(self.artist, self.title)] = val
			return val

	def _info(self, lfm_link, fallback=dict(date=None, link=None, summary=None)):
		'Cached query to last.fm database'
		from pylast import WSError

		# Query cache by (artist,title)
		try: return self._info_cache()
		except KeyError: pass

		try:
			try: link = lfm_link.get_album_by_mbid(self.mbdb_id)
			except WSError:
				try: link = lfm_link.get_album(self.artist, self.title)
				except WSError: return self._info_cache(val=fallback)
		except HTTPException as err:
			log.error('HTTP error from last.fm: {0}'.format(err))
			raise AssertionError

		# Query cache by last.fm id
		try: return self._info_cache(lfm_id=link.get_id())
		except KeyError: pass
		except WSError: return self._info_cache(val=fallback)

		rel_date = link.get_release_date() or None
		if rel_date: rel_date = date(*parsedate(rel_date)[:3])
		return self._info_cache(lfm_id=link.get_id(), val=dict(
			date=rel_date, link=link.get_url(), summary=link.get_wiki_summary() ))

